import datetime
import os

import authlib.integrations.requests_client
import authlib.jose
import authlib.oidc.core
import flask
import requests
import werkzeug.exceptions

oidc_blueprint = flask.Blueprint("oidc", __name__, url_prefix="/oidc")


def generate_session() -> authlib.integrations.requests_client.OAuth2Session:
    realm_url = flask.current_app.config["OIDC_RP_REALM_URL"]

    return authlib.integrations.requests_client.OAuth2Session(
        client_id=flask.current_app.config["OIDC_RP_CLIENT_ID"],
        client_secret=flask.current_app.config["OIDC_RP_CLIENT_SECRET"],
        authorization_endpoint=f"{realm_url}protocol/openid-connect/auth",
        token_endpoint=f"{realm_url}protocol/openid-connect/token",
        userinfo_endpoint=f"{realm_url}protocol/openid-connect/userinfo",
        jwks_uri=f"{realm_url}protocol/openid-connect/certs",
        scope="openid profile groups",
    )


def generate_redirect_uri() -> str:
    return flask.url_for(
        "oidc.authenticate",
        _external=True,
        _scheme=(
            flask.request.headers.get(
                "X-Forwarded-Proto",
                "http" if flask.request.url.startswith("http://") else "https",
            )
        ),
    )


@oidc_blueprint.route("/login")
def login() -> flask.Response:
    with generate_session() as session:
        url, state = session.create_authorization_url(
            session.metadata["authorization_endpoint"],
            redirect_uri=generate_redirect_uri(),
        )

        return flask.redirect(url)


@oidc_blueprint.route("/authenticate")
def authenticate() -> flask.Response:
    code = flask.request.args.get("code")
    state = flask.request.args.get("state")

    if code is None or state is None:
        raise werkzeug.exceptions.BadRequest

    with generate_session() as session:
        try:
            token = session.fetch_token(
                session.metadata["token_endpoint"],
                code=code,
                state=state,
                redirect_uri=generate_redirect_uri(),
            )
        except (
            authlib.integrations.requests_client.OAuthError,
            requests.HTTPError,
        ) as exception:
            flask.current_app.logger.debug(
                "Error during OIDC authentication: %s", exception
            )

            raise werkzeug.exceptions.Unauthorized from exception

        jwks = session.get(session.metadata["jwks_uri"]).json()

        claims = authlib.jose.jwt.decode(
            token["id_token"], jwks, claims_cls=authlib.oidc.core.CodeIDToken
        )
        claims.validate()

        userinfo = session.get(session.metadata["userinfo_endpoint"]).json()

    current_timestamp = round(datetime.datetime.now().timestamp())

    # found_allowed_group = False

    # for group in claims["groups"]:
    # if group in flask.current_app.config["ALLOWED_OIDC_GROUPS"]:
    # found_allowed_group = True
    # break

    # if not found_allowed_group:
    # raise werkzeug.exceptions.Unauthorized(
    # "Nemáš dostatečné skupiny k přístupu do Generátoru."
    # )

    jwt = authlib.jose.jwt.encode(
        {"alg": "HS256"},
        {
            "iss": "graphics-generator",
            "iat": current_timestamp,
            "exp": (current_timestamp + flask.current_app.config["JWT_EXPIRES_AFTER"]),
            "sub": userinfo["sub"],  # Currently unused, maybe of use later
        },
        flask.current_app.config["JWT_SECRET_KEY"],
    ).decode("utf-8")

    response = flask.make_response(flask.redirect(flask.url_for("frontend.index")))

    response.set_cookie("token", jwt)

    return response
